
from zope.interface import Interface


class IForum(Interface):
    """forum content"""

class IForumTopic(Interface):
    """forum topic content"""

class IForumPost(Interface):
    """forum post content"""

class IForumNotifiable(Interface):
    """ forum notifiable """

class IForumNotification(Interface):
    """ forim notification """




# -*- coding: utf-8 -*-

from zope.component import getMultiAdapter
from os.path import basename
from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class BrowserContentView(BrowserView):
    """ Content Provider for main content area.
        Now its possible that we reload main content area with KSS. """

    __call__ = ViewPageTemplateFile('content_view.pt')
    render = None

    def __init__(self, context, request, view=None):
        self.context = aq_inner(context)
        self.request = request
        if view: 
            self.__parent__ = view

        self.portal_state = getMultiAdapter((self.context, self.request), 
                                                    name=u'plone_portal_state')
        self.site_url = self.portal_state.portal_url()
    
    def update(self):
        """ method that is being call when content provider"""
        pass

    def getPhysicalPath(self):
        """ needed so its possible to do path search over catalog 
              ExtendedPathIndex needs it ... maybe a bug  """
        return self.context.getPhysicalPath()

    def render_template(self):
        """ return name of render template """
        return basename(self.render.filename)[:-3]
    



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class ForumPostView(BrowserView):

    def __call__(self, *argv, **kw):
        context = aq_inner(self.context)
        request = self.request

        parent = context.getParentNode()
        redirect_url = parent.absolute_url()+'#'+context.UID()

        return request.RESPONSE.redirect(redirect_url)





# -*- coding: utf-8 -*-

from zope.interface import implements
from AccessControl import ClassSecurityInfo

from Products.Archetypes import atapi
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from garbas.forum.content.base import ForumFolder
from garbas.forum.content.base import ForumFolderSchema
from garbas.forum.interfaces import IForum
from garbas.forum.config import PROJECTNAME
from garbas.forum import ForumMessageFactory as _


ForumSchema = ForumFolderSchema.copy() + atapi.Schema((
    atapi.LinesField(
        name='categories',
        required=False,
        storage = atapi.AnnotationStorage(),
        widget=atapi.LinesWidget(
            label = _(u'categories_label', default=u'Categories'),
            description = _(u'categories_help', default='Write categories (one in each line) in order of appearing.'),
        ),
    ),
    atapi.StringField(
        name='category',
        required=False,
        storage = atapi.AnnotationStorage(),
        widget=atapi.SelectionWidget(
            label = _(u'category_label', default=u'Category'),
            description = _(u'category_help', default='Select category in which forum you are just creating, should appear.'),
            format='select',
        ),
        vocabulary='getAvaliableCategories',
    ),
    atapi.BooleanField(
        name='allow_addtopic',
        default=True,
        storage = atapi.AnnotationStorage(),
        widget=atapi.BooleanWidget(
            label = _(u'allow_addtopic_label', default=u'Allow to add topic'),
            description = _(u'allow_addtopic_help', default='If checked you are allowing user to add topic to forum. This option is useful when you are creating forum container and you dont want to have any topic inside your container.'),
        ),
    ),
))


class Forum(ForumFolder):
    """forum content"""

    implements(IForum)

    portal_type = "Forum"
    schema = ForumSchema
    security = ClassSecurityInfo()
    
    title       = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    categories = atapi.ATFieldProperty('categories')
    category = atapi.ATFieldProperty('category')
    allow_addtopic = atapi.ATFieldProperty('allow_addtopic')

    security.declarePublic('getAvaliableCategories')
    def getAvaliableCategories(self):
        parent = self.getParentNode()
        avaliable_categories = [_(u'no_category', default=u'No category'),]
        if parent.portal_type == 'Forum':
            for category in parent.categories:
                avaliable_categories.append(category.decode('utf-8'))
        return avaliable_categories

atapi.registerType(Forum, PROJECTNAME)





"""Initializer used when used as a Zope 2 product
"""

from Products.Archetypes import atapi
from Products.CMFCore import utils

from zope.i18nmessageid import MessageFactory
ForumMessageFactory = MessageFactory('garbas.forum')

from garbas.forum import config
from garbas.forum.permissions import initialize_permissions


def initialize(context):
    """Initializer called when used as a Zope 2 product."""

    initialize_permissions()

    from content import forum, topic, post
    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)
    
    for atype, constructor in zip(content_types, constructors):
        utils.ContentInit("%s: %s" % (config.PROJECTNAME, atype.portal_type),
            content_types      = (atype,),
            permission         = config.ADD_PERMISSIONS[atype.portal_type],
            extra_constructors = (constructor,),
            ).initialize(context)





## Script (Python) "contentUsesKupu"
##title=Allow graceful degradation if content is not text/html
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=fieldname
from Products.CMFCore.utils import getToolByName

from Products.CMFCore.utils import getToolByName
tool = getToolByName(context, 'kupu_library_tool')
REQUEST = context.REQUEST

# If the user doesn't have kupu configured then we can't use it.
if not tool.isKupuEnabled(allowAnonymous=True,
                          REQUEST=REQUEST,
                          context=context,
                          fieldName=fieldname):
    return False

if not fieldname:
    return True # Non AT content always tries to use kupu

if fieldname == REQUEST.form.get('kupu.convert', ''):
    return True
if fieldname == REQUEST.form.get('kupu.suppress', ''):
    return False

if not hasattr(context, 'getField'):
    return True
    
field = context.getField(fieldname)
if not field:
  return True
text_format = REQUEST.get('%s_text_format' % fieldname, context.getContentType(fieldname))
content = field.getEditAccessor(context)()
try:
    if content.startswith('<!--'):
        return False
except AttributeError:
    return False
return len(content)==0 or 'html' in text_format.lower()




# -*- coding: utf-8 -*-

from datetime import datetime

from Acquisition import aq_inner

from zope.interface import alsoProvides
from zope.component import getMultiAdapter
from zope.component import getUtility

from kss.core import kssaction
from plone.memoize.instance import memoize
from plone.app.layout.viewlets import common
from plone.app.kss.plonekssview import PloneKSSView
from plone.app.layout.globals.interfaces import IViewView
from plone.i18n.normalizer.interfaces import IUserPreferredURLNormalizer
from collective.captcha.browser.captcha import Captcha

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone import Batch

from email.MIMEMultipart import MIMEMultipart
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.Utils import COMMASPACE, formatdate
from email import Encoders

from garbas.forum.browser import BrowserContentView
from garbas.forum.interfaces import IForum
from garbas.forum.interfaces import IForumTopic
from garbas.forum.interfaces import IForumNotification
from garbas.forum import ForumMessageFactory as _


TOPIC_SUCCESS     = _(u'topic_success', default='Topic was successfuly added.')
TOPIC_TITLE_ERROR = _(u'topic_title_error', default='Subject is required. Please correct.')
TOPIC_TEXT_ERROR  = _(u'topic_text_error', default='Text is required. Please correct.')
CAPTCHA_ERROR    = _(u'captcha_error', default=u'Captcha is required. Please correct.')
CAPTCHA_ERROR2   = _(u'captcha_error2', default=u'Captcha is incorrect.')


class ForumView(BrowserContentView):
    """ forum main view """

    render = ViewPageTemplateFile('templates/forum.pt')

    def render_template(self):
        return 'garbas.forum.forum_view' 
  
    def has_permissions_to_addtopic(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.checkPermission("Forum: Submit New Post", self.context) and \
                  self.context.allow_addtopic

    def has_topics(self):
      return len(self.topics()) > 0

    def has_subforums(self):
      return len(self.subforums()) > 0

    @memoize
    def topics(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return [dict(
                 title          = topic.Title,
                 url            = topic.getURL(),
                 author         = topic.Creator,
                 created        = topic.created,
                 replies        = topic.forum_posts,
                 last_author    = len(topic.forum_info)==2 and topic.forum_info[0] or '',
                 last_post_url  = len(topic.forum_info)==2 and self.context.absolute_url()+\
                                                               '/'+topic.getId+\
                                                               topic.forum_info[1] or '',
                 last_post_date = topic.modified,)
            for topic in catalog(
                 object_provides = IForumTopic.__identifier__,
                 path = dict(query='/'.join(self.context.getPhysicalPath()),
                             depth=1),
                 sort_on = getattr(self.request, 'topic.sort_on', 'modified'),
                 sort_order = getattr(self.request, 'topic.sort_order', 'reverse'))]
    
    def topics_batch(self):
        return Batch(self.topics(),
            getattr(self.request, 'b_size', 20),
            getattr(self.request, 'b_start', 0),
            orphan=1)


    @memoize
    def subforums(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        uncategorizes_forums = _(u'uncategorizes_forums', default='Uncategorized')

        sorting = getattr(self.request, 'subforum.sort_on', 'modified')
        sorting_order = getattr(self.request, 'subforum.sort_order', 'reverse')

        forums = {}
        for forum in catalog(object_provides=IForum.__identifier__,
                             path=dict(query='/'.join(self.context.getPhysicalPath()),depth=1),
                             sort_on=sorting,
                             sort_order=sorting_order,):
            if forum.forum_category in self.context.categories:
                category = forum.forum_category
            else:
                category = uncategorizes_forums
            if not forums.has_key(category):
                forums[category] = []
            forums[category].append(dict(
                title           = forum.Title,
                url             = forum.getURL(),
                description     = forum.Description,
                topics          = forum.forum_topics,
                posts           = forum.forum_posts+forum.forum_topics,
                last_author     = len(forum.forum_info)==2 and forum.forum_info[0] or '',
                last_post_url   = len(forum.forum_info)==2 and self.context.absolute_url()+\
                                                               '/'+forum.getId+\
                                                               forum.forum_info[1] or '',
                last_post_date  = forum.modified))
        categories = self.context.categories + (uncategorizes_forums, )
        subforums = []
        for category in categories:
            if not forums.has_key(category): continue
            subforums.append(dict(
                title=category,
                forums=forums[category],))
        return subforums
           

class ForumForm(object):

    def validate(self):
        errors = {}
        for method in dir(self):
            if method[:9] == 'validate_':
                method = getattr(self, method, False)
                if method:
                    tmp_err = method()
                    if tmp_err:
                        errors.update(tmp_err)
        return errors

    def generateNewId(self, title):
        URLNormalizer = IUserPreferredURLNormalizer(self.request)
        id = URLNormalizer.normalize(title)
        def findId(id, i=1):
            if getattr(self.context, id+'-'+str(i), 'expected')!='expected':
                return findId(id, i+1)
            else:
                return id+'-'+str(i)
        if getattr(self.context, id, 'expected')!='expected':
            id = findId(id)
        return id

    def notifySubscribers(self, subject, body):
        host = self.context.MailHost
        notification = IForumNotification(self.context)
        portal = getUtility(ISiteRoot)
        mail_encoding = portal.getProperty('email_charset')
        portal_from = '"' + str(portal.getProperty('email_from_name')) + \
                      '" <' + str(portal.getProperty('email_from_address')) + '>'
        
        mail = MIMEMultipart()
        mail.set_charset(mail_encoding)
        mail['From'] = portal_from
        mail['Date'] = formatdate(localtime=True)
        mail['Subject'] = subject.encode(mail_encoding)
        mail.attach(MIMEText(body.encode(mail_encoding), 'html'))
        
        for email in notification.email_list():
            mail['To'] = email
            host.send(mail.as_string().encode(mail_encoding))


class ForumAddTopic(BrowserContentView, ForumForm):
    """ add topic form """
    
    errors = dict()
    render = ViewPageTemplateFile('templates/add_topic.pt')
   
    def render_template(self):
        return 'garbas.forum.add_topic' 

    def update(self):
        """ all action happens here """

        if self.request.get('form.button.cancel', None):
            self.request.response.redirect(self.context.absolute_url())
        elif self.request.get('form.button.submit', None):
            self.errors = self.validate()
            if not self.errors:
                membership = getToolByName(self.context, 'portal_membership')
                utils = getToolByName(self.context, 'plone_utils')
                transforms = getToolByName(self.context, 'portal_transforms')
                
                types = getToolByName(self.context, 'portal_types')
                factory = types.getTypeInfo('ForumTopic')
                if factory is None:
                    raise Exception, 'Wrong type defined.'
                
                topic_title = getattr(self.request, 'topic_title', None)
                topic_id = self.generateNewId(topic_title)
                topic_text = getattr(self.request, 'topic_text', None)
                topic_text = transforms('text_to_html', topic_text)
                topic_author = membership.getAuthenticatedMember()

                topic = factory._constructInstance(self.context, topic_id)
                topic.title = topic_title
                topic.text  = topic_text
                factory._finishConstruction(topic)

                if self.request.get('notify_me', None)=='1':
                    IForumNotification(topic).subscribe(topic_author.getProperty('email'))

                utils.addPortalMessage(TOPIC_SUCCESS, request=self.request)
                self.request.response.redirect(topic.absolute_url())
    
    def validate_title(self):
        if 'topic_title' not in self.request or \
           not self.request['topic_title']:
            self.errors['topic_title'] = TOPIC_TITLE_ERROR

    def validate_text(self):
        if 'topic_text' not in self.request or \
           not self.request['topic_text']:
            self.errors['topic_text'] = TOPIC_TEXT_ERROR

    def validate_captcha(self):
        if not self.is_anonymous(): return
        if 'captcha' not in self.request or \
           not self.request['captcha']:
            self.errors['captcha'] = CAPTCHA_ERROR
        elif not Captcha(self.context, self.request).verify(self.request['captcha']):
            self.errors['captcha'] = CAPTCHA_ERROR2
        
    def is_anonymous(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.isAnonymousUser()

    def captcha(self):
        return Captcha(self.context, self.request)






# -*- coding: utf-8 -*-

from zope.interface import implements

from Products.Archetypes import atapi

from Products.ATContentTypes.content.document import ATDocumentSchema
from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from garbas.forum.config import PROJECTNAME

from garbas.forum import ForumMessageFactory as _
from garbas.forum.interfaces import IForumPost



PostSchema = ATDocumentSchema.copy() + atapi.Schema((
    atapi.TextField(
        name='text',
        storage=atapi.AnnotationStorage(),
        searchable=True,
        # TODO :: default = 'getDefaultText', should be aware of quoting
        default_output_type = 'text/x-html-safe',
        widget = atapi.TextAreaWidget( # TODO :: BBCode widget
            label = _(u'topic_label_text', default=u'Text'),
            description = _(u'topic_help_text', default='Write text.'),
        ),
    ),
))

PostSchema['title'].storage = atapi.AnnotationStorage()
PostSchema['title'].default = 'getDefaultTitle'
PostSchema['description'].storage = atapi.AnnotationStorage()
PostSchema['description'].widget.visible = {'edit':'invisible', 'view':'invisible'}

finalizeATCTSchema(PostSchema)


class ForumPost(ATDocument):
    """forum post content
    """
    implements(IForumPost)
    
    portal_type = "ForumPost"
    _at_rename_after_creation = True
    schema = PostSchema
    
    title = atapi.ATFieldProperty('title')
    text = atapi.ATFieldProperty('text')

    def getDefaultTitle(self):
        return 'RE: ' + self.getParentNode().Title()
   
atapi.registerType(ForumPost, PROJECTNAME)







from Acquisition import aq_inner

from BTrees.OOBTree import OOBTree
from persistent.list import PersistentList
from zope.interface import implements
from zope.annotation.interfaces import IAnnotations

from Products.CMFCore.utils import getToolByName

from garbas.forum.interfaces import IForumNotification


NOTIFICATION_KEY = 'garbas.forum.notification'

class ForumNotification(object):
    """ notification """

    implements(IForumNotification)

    def __init__(self, context):
        self.context = aq_inner(context)
        self.annotations = IAnnotations(self.context)
        notifications = self.annotations.get(NOTIFICATION_KEY, None)
        if notifications is None:
            self.annotations[NOTIFICATION_KEY] = PersistentList()

    def email_list(self, ignore=[]):
        email_list = []
        membership = getToolByName(self.context, 'portal_membership')
        for email in self.annotations[NOTIFICATION_KEY]:
            if email not in ignore:
                try:
                    ploneMember = membership.getMemberById(email)
                except:
                    ploneMember = None
                if ploneMember:
                    ploneMemberEmail = ploneMember.getProperty('email')
                    if ploneMemberEmail:
                        email = ploneMemberEmail
                # TODO :: check if its valid email before appending it
                email_list.append(email)
        return email_list

    def subscribe(self, email):
        if email not in self.annotations[NOTIFICATION_KEY]:
            if email:
                self.annotations[NOTIFICATION_KEY].append(email)
    
    def unsubscribe(self, email):
        if email in self.annotations[NOTIFICATION_KEY]:
            self.annotations[NOTIFICATION_KEY].remove(email)






# -*- coding: utf-8 -*-

from zope.i18n import translate
from zope.event import notify
from zope.component import getUtility
from zope.component import getMultiAdapter
from zope.lifecycleevent import ObjectModifiedEvent
from plone.memoize.instance import memoize
from plone.i18n.normalizer.interfaces import IUserPreferredURLNormalizer
from collective.captcha.browser.captcha import Captcha

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import Batch
from Products.CMFCore.interfaces import ISiteRoot

from garbas.forum.browser import BrowserContentView
from garbas.forum.browser.forum import ForumForm
from garbas.forum.interfaces import IForumPost
from garbas.forum.interfaces import IForumNotification
from garbas.forum import ForumMessageFactory as _

from lajf.login.interfaces import IMember

POST_SUCCESS        = _(u'post_success', default='Post was successfuly added.')
POST_TITLE_ERROR    = _(u'post_title_error', default='Subject is required. Please correct.')
POST_TEXT_ERROR     = _(u'post_text_error', default='Text is required. Please correct.')
CAPTCHA_ERROR       = _(u'captcha_error', default=u'Captcha is required. Please correct.')
CAPTCHA_ERROR2      = _(u'captcha_error2', default=u'Captcha is incorrect.')
UNSUBSCRIBE_SUCCESS = _(u'unsubscribe_success', default=u'Successfully unsubscribed.')
UNSUBSCRIBE_ERROR   = _(u'unsubscribe_error', default=u'There was error when unsubscribing. Please contact administrator.')
SUBSCRIBE_SUCCESS   = _(u'subscribe_success', default=u'Successfully subscribed.')
SUBSCRIBE_ERROR     = _(u'subscribe_error', default=u'There was error when subscribing. Please contact administrator.')

class ForumTopicView(BrowserContentView):
    """ topic view """

    errors = dict()
    render = ViewPageTemplateFile('templates/topic.pt')
   
    def render_template(self):
        return 'garbas.forum.topic_view' 

    def update(self):
        utils = getToolByName(self.context, 'plone_utils')
        membership = getToolByName(self.context, 'portal_membership')
        member = membership.getAuthenticatedMember()

        if self.request.get('subscribe', None)=='1':
            if member:
                IForumNotification(self.context).subscribe(member.getProperty('email'))
                utils.addPortalMessage(SUBSCRIBE_SUCCESS, request=self.request)
            else:
                utils.addPortalMessage(SUBSCRIBE_ERROR, 'error', request=self.request)
            self.request.response.redirect(self.context.absolute_url())
        elif self.request.get('unsubscribe', None)=='1':
            if member:
                IForumNotification(self.context).unsubscribe(member.getProperty('email'))
                utils.addPortalMessage(UNSUBSCRIBE_SUCCESS, request=self.request)
            else:
                utils.addPortalMessage(UNSUBSCRIBE_ERROR, 'error', request=self.request)
            self.request.response.redirect(self.context.absolute_url())
    
    @memoize
    def subscribe_list(self):
        return IForumNotification(self.context).email_list()

    def is_subscribed(self):
        membership = getToolByName(self.context, 'portal_membership')
        member = membership.getAuthenticatedMember()
        return member.getProperty('email') in self.subscribe_list()

    @memoize
    def has_permissions_to_addposts(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.checkPermission("Forum: Submit New Post", self.context)

    def has_posts(self):
        return len(self.posts()) > 0

    @memoize
    def posts(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        membership = getToolByName(self.context, 'portal_membership')

        b_size = getattr(self.request, 'b_size', 20)
        b_start = getattr(self.request, 'b_start', 0)

        result = [ dict( title        = item.Title,
                         UID          = item.UID,
                         author       = item.Creator and item.Creator or '',
                         portrait_url = membership.getPersonalPortrait(item.Creator).absolute_url(),
                         created      = item.created,
                         text         = item.getObject().text ) # TODO :: maybe put it in catalog
                   for item in catalog(
                         object_provides = IForumPost.__identifier__,
                         path            = dict(query='/'.join(self.context.getPhysicalPath()),
                                                depth=1),
                         sort_on         = 'modified',)]

        return Batch([dict( title        = self.context.title,
                            UID          = self.context.UID(),
                            author       = self.context.Creator(),
                            portrait_url = membership.getPersonalPortrait(self.context.Creator()).absolute_url(),
                            created      = self.context.CreationDate(),
                            text         = self.context.text )] + result, b_size, b_start, orphan=1)


class ForumAddPost(BrowserContentView, ForumForm):
    """ add post form """
    
    errors = dict()
    render = ViewPageTemplateFile('templates/add_post.pt')
   
    def render_template(self):
        return 'garbas.forum.add_post' 

    def update(self):
        """ all action happens here """
        
        if self.request.get('form.button.cancel', None):
            self.request.response.redirect(self.context.absolute_url())
        elif self.request.get('form.button.submit', None):
            self.errors = self.validate()
            if not self.errors:
                membership = getToolByName(self.context, 'portal_membership')
                utils = getToolByName(self.context, 'plone_utils')
                transforms = getToolByName(self.context, 'portal_transforms')
                types = getToolByName(self.context, 'portal_types')
                factory = types.getTypeInfo('ForumPost')
                if factory is None:
                    raise Exception, 'Wrong type defined.'
                
                post_title = getattr(self.request, 'post_title', None)
                post_id = self.generateNewId(post_title)
                post_text = getattr(self.request, 'post_text', None)
                post_text = transforms('text_to_html', post_text)
                post_author = membership.getAuthenticatedMember()
                
                post_author_name = ''
                if post_author:
                    post_author_name = post_author.getProperty('fullname')

                post = factory._constructInstance(self.context, post_id)
                post.title = post_title
                post.text  = post_text
                factory._finishConstruction(post)

                self.notifySubscribers(
                    subject = translate(_(u'mail_notify_change_subject',
                        default=u'New reply on ${topic_title}',
                        mapping=dict(topic_title=self.context.Title()))),
                    body    = translate(_(u'mail_notify_change_text',
                        default='''
                            <a href="${post_url}">
                                <b>${post_title}</b>
                            </a><br />by ${post_author}
                            <p>${post_text}</p>
                            <br />
                            --- Unsubscribe here: <a href="${unsubscribe}">${unsubscribe}</a>''',
                        mapping=dict(post_title=post_title,
                                     post_url=post.absolute_url(),
                                     post_author=post_author_name,
                                     post_text=post_text,
                                     unsubscribe=self.context.absolute_url()+'/?unsubscribe=1'))))
                
                if self.request.get('notify_me', None)=='1':
                    IForumNotification(self.context).subscribe(post_author.getProperty('email'))

                utils.addPortalMessage(POST_SUCCESS, request=self.request)
                self.request.response.redirect(post.absolute_url())

    def validate_title(self):
        if 'post_title' not in self.request or \
           not self.request['post_title']:
            self.errors['post_title'] = POST_TITLE_ERROR

    def validate_text(self):
        if 'post_text' not in self.request or \
           not self.request['post_text']:
            self.errors['post_text'] = POST_TEXT_ERROR

    def validate_captcha(self):
        membership = getToolByName(self.context, 'portal_membership')
        if not membership.isAnonymousUser(): return
        if 'captcha' not in self.request or \
           not self.request['captcha']:
            self.errors['captcha'] = CAPTCHA_ERROR
        elif not Captcha(self.context, self.request).verify(self.request['captcha']):
            self.errors['captcha'] = CAPTCHA_ERROR2
    
    def is_anonymous(self):
        membership = getToolByName(self.context, 'portal_membership')
        return membership.isAnonymousUser()

    def captcha(self):
        return Captcha(self.context, self.request)

    @memoize
    def subscribe_list(self):
        return IForumNotification(self.context).email_list()

    def is_subscribed(self):
        membership = getToolByName(self.context, 'portal_membership')
        member = membership.getAuthenticatedMember()
        return member.getProperty('email') in self.subscribe_list()





# -*- coding: utf-8 -*-

from zope.interface import implements

from Products.Archetypes import atapi
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.CMFCore.exceptions import AccessControl_Unauthorized

from garbas.forum.content.base import ForumFolder
from garbas.forum.content.base import ForumFolderSchema
from garbas.forum.interfaces import IForumTopic
from garbas.forum.interfaces import IForum
from garbas.forum.config import PROJECTNAME
from garbas.forum import ForumMessageFactory as _


TopicSchema = ForumFolderSchema.copy() + atapi.Schema((
    atapi.TextField(
        name='text',
        required = True,
        searchable=True,
        default_output_type = 'text/x-html-safe',
        storage=atapi.AnnotationStorage(),
        widget = atapi.TextAreaWidget( # TODO :: BBCode widget
            label = _(u'topic_label_text', default=u'Text'),
            description = _(u'topic_help_text', default='Write text.'),
        ),
    ),
))
TopicSchema['description'].widget.visible = {'edit':'invisible', 'view':'invisible'}


class ForumTopic(ForumFolder):
    """forum topic content"""

    implements(IForumTopic)

    portal_type = "ForumTopic"
    schema = TopicSchema
    
    title       = atapi.ATFieldProperty('title')
    text        = atapi.ATFieldProperty('text')
   
atapi.registerType(ForumTopic, PROJECTNAME)






import doctest
import unittest

from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup


@onsetup
def setup_product():
    fiveconfigure.debug_mode = True
    import garbas.forum
    zcml.load_config('configure.zcml', garbas.forum)
    fiveconfigure.debug_mode = False
    ztc.installPackage('garbas.forum')

setup_product()
ptc.setupPloneSite(products=['garbas.forum'])


class FunctionalTestCase(ptc.FunctionalTestCase):
    """ functional test case """


def test_suite():
    return unittest.TestSuite([
        ztc.ZopeDocFileSuite('content.txt',
            package='garbas.forum', test_class=FunctionalTestCase,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE |
                        doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
        ztc.ZopeDocFileSuite('notification.txt',
            package='garbas.forum', test_class=FunctionalTestCase,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE |
                        doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
        ztc.ZopeDocFileSuite('browser.txt',
            package='garbas.forum', test_class=FunctionalTestCase,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE |
                        doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# -*- coding: utf-8 -*-

from DateTime import DateTime
from Acquisition import aq_inner
from zope.interface import implements
from zope.component import getMultiAdapter
from AccessControl import ClassSecurityInfo

from Products.Archetypes import atapi

from Products.CMFCore.utils import getToolByName
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from garbas.forum import ForumMessageFactory as _
from garbas.forum.interfaces import IForumTopic
from garbas.forum.interfaces import IForumPost
from garbas.forum.interfaces import IForum


ForumFolderSchema = ATBTreeFolderSchema.copy()
ForumFolderSchema['title'].storage = atapi.AnnotationStorage()
ForumFolderSchema['description'].storage = atapi.AnnotationStorage()


class ForumFolder(ATBTreeFolder):
    """forum folder"""

    _at_rename_after_creation = True
    schema = ForumFolderSchema

    def _getLastPost(self):
        catalog = getToolByName(self, 'portal_catalog')
        result = catalog(
            object_provides = IForumPost.__identifier__,
            path            = dict(query='/'.join(self.getPhysicalPath())),
            sort_on         = 'created',
            sort_order      = 'reverse',
            sort_limit      = 1,)
        if result:
            return result[0].getObject()

    def _getLastTopic(self):
        catalog = getToolByName(self, 'portal_catalog')
        result = catalog(
            object_provides = IForumTopic.__identifier__,
            path            = dict(query='/'.join(self.getPhysicalPath())),
            sort_on         = 'created',
            sort_order      = 'reverse',
            sort_limit      = 1,)
        if result:
          return result[0].getObject()

    # CATALOG INDEXES
    def forum_category(self):
        """forum category"""
        return getattr(self, 'category', '')
            
    def forum_topics(self):
        """number of all topics"""
        catalog = getToolByName(self, 'portal_catalog')
        return len(catalog(
            object_provides = IForumTopic.__identifier__,
            path            = dict(query='/'.join(self.getPhysicalPath()))))
            
    def forum_posts(self):
        """number of all posts"""
        catalog = getToolByName(self, 'portal_catalog')
        return len(catalog(
            object_provides = IForumPost.__identifier__,
            path            = dict(query='/'.join(self.getPhysicalPath()))))
            
    def modified(self):
        """last post date"""
        last_post = self._getLastPost()
        if not last_post:
            last_post = self._getLastTopic()
        return last_post!=None and last_post.created() or self.created()

    def forum_info(self):
        last_post = self._getLastPost()
        if not last_post:
            last_post = self._getLastTopic()
        return [last_post!=None and last_post.Creator() or '',
                last_post!=None and last_post.absolute_url()[len(self.absolute_url()):] or '']


def reindex_for_topics(context, event):
    """reindex all sub-forum folders"""
    def reindex_parent(context):
        parent = aq_inner(context.getParentNode())
        if IForum.providedBy(parent) or \
           IForumTopic.providedBy(parent):
            parent.reindexObject(['forum_topics',
                                  'modified',
                                  'forum_info',])
            reindex_parent(parent)
    reindex_parent(context)

def reindex_for_posts(context, event):
    """reindex all sub-forum folders"""
    def reindex_parent(context):
        parent = aq_inner(context.getParentNode())
        if IForum.providedBy(parent) or \
           IForumTopic.providedBy(parent):
            parent.reindexObject(['forum_posts',
                                  'modified',
                                  'forum_info',])
            reindex_parent(parent)
    reindex_parent(context)




import os
from setuptools import setup, find_packages

version = '0.1b9'

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup(name='garbas.forum',
      version=version,
      description="Forum for Plone",
      long_description=('\n'.join((
              read('README.txt'), '',
              read('TODO.txt'), '',
              read('HISTORY.txt'), '',
          ))),

      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Rok Garbas',
      author_email='rok.garbas@gmail.com',
      url='http://github.com/garbas/garbas.forum/tree/master',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['garbas'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'collective.captcha',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
